package org.gcube.dataharvest.harvester.sobigdata;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.SortedSet;

import org.apache.commons.lang.Validate;
import org.gcube.common.homelibrary.home.Home;
import org.gcube.common.homelibrary.home.HomeLibrary;
import org.gcube.common.homelibrary.home.HomeManager;
import org.gcube.common.homelibrary.home.exceptions.InternalErrorException;
import org.gcube.common.homelibrary.home.workspace.WorkspaceItem;
import org.gcube.common.homelibrary.home.workspace.accounting.AccountingEntry;
import org.gcube.common.homelibrary.jcr.repository.JCRRepository;
import org.gcube.common.homelibrary.jcr.workspace.JCRWorkspace;
import org.gcube.common.homelibrary.jcr.workspace.JCRWorkspaceItem;
import org.gcube.dataharvest.datamodel.HarvestedData;
import org.gcube.dataharvest.utils.DateUtils;
import org.gcube.dataharvest.utils.Utils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author Eric Perrone (ISTI - CNR)
 * @author Luca Frosini (ISTI - CNR)
 */
public class DataMethodDownloadHarvester extends SoBigDataHarvester {
	
	private static Logger logger = LoggerFactory.getLogger(DataMethodDownloadHarvester.class);
	
	private int count = 0;
	
	public DataMethodDownloadHarvester(Date start, Date end, SortedSet<String> contexts) throws ParseException {
		super(start, end, contexts);
	}
	
	@Override
	public List<HarvestedData> getData() throws Exception {
		String context = Utils.getCurrentContext();
		
		try {
			ArrayList<HarvestedData> data = new ArrayList<HarvestedData>();
			
			count = 0;
			
			HomeManager manager = HomeLibrary.getHomeManagerFactory().getHomeManager();
			
			String user = getVREName(context) + "-Manager";
			
			@SuppressWarnings("deprecation")
			Home home = manager.getHome(user);
			
			JCRWorkspace ws = (JCRWorkspace) home.getWorkspace();
			
			JCRWorkspaceItem item = (JCRWorkspaceItem) ws
					.getItemByPath("/Workspace/MySpecialFolders/" + getVREName(context));
			
			logger.debug("Analyzing " + context + " from " + startDate.toString() + " to " + endDate.toString());
			
			logger.error("Before getStats()");
			getStats(item, startDate, endDate);
			logger.error("After getStats()");
			HarvestedData harvest = new HarvestedData(HarvestedData.DATA_METHOD_DOWNLOAD, context, count);
			data.add(harvest);
			logger.debug(harvest.toString());
			return data;
			
		} catch(Exception e) {
			throw e;
		}
		
	}
	
	private void getStats(WorkspaceItem root, Date start, Date end) throws InternalErrorException {
		List<? extends WorkspaceItem> children;
		if(root.isFolder()) {
			children = root.getChildren();
			for(WorkspaceItem child : children)
				getStats(child, start, end);
		} else {
			try {
				
				List<AccountingEntry> accounting = root.getAccounting();
				for(AccountingEntry entry : accounting) {
					
					switch(entry.getEntryType()) {
						case CREATE:
						case UPDATE:
						case READ:
							Calendar calendar = entry.getDate();
							if(calendar.after(DateUtils.dateToCalendar(start))
									&& calendar.before(DateUtils.dateToCalendar(end))) {
								count++;
							}
							
							break;
						
						default:
							break;
					}
					
				}
			} catch(Exception e) {
				logger.error("DataMethodDownloadHarvester: " + e.getLocalizedMessage());
				throw new InternalErrorException(e.getLocalizedMessage());
			}
		}
	}
	
	private static String getVREName(String vre) {
		Validate.notNull(vre, "scope must be not null");
		
		String newName;
		if(vre.startsWith(JCRRepository.PATH_SEPARATOR))
			newName = vre.replace(JCRRepository.PATH_SEPARATOR, "-").substring(1);
		else
			newName = vre.replace(JCRRepository.PATH_SEPARATOR, "-");
		return newName;
	}
	
}
